{% block _text_id_widget -%}
    <div id="container">
        {{- form_widget(form) -}}
    </div>
{%- endblock _text_id_widget %}

{% block _names_entry_label -%}
    {% if label is empty %}
        {%- set label = name|humanize -%}
    {% endif -%}
    <label>Custom label: {{ label|trans(label_translation_parameters, translation_domain) }}</label>
{%- endblock _names_entry_label %}

{% block _name_c_entry_label -%}
    {% if label is empty %}
        {%- set label = name|humanize -%}
    {% endif -%}
    <label>Custom name label: {{ label|trans(label_translation_parameters, translation_domain) }}</label>
{%- endblock _name_c_entry_label %}
