<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\HttpFoundation\Tests\Session\Storage\Handler;

/**
 * @group integration
 */
class RedisSessionHandlerTest extends AbstractRedisSessionHandlerTestCase
{
    /**
     * @return \Redis|object
     */
    protected function createRedisClient(string $host)
    {
        $client = new \Redis();
        $client->connect($host);

        return $client;
    }
}
