{% deprecated 'The template "' ~ _self ~'" is deprecated since Symfony 4.4, will be removed in 5.0.' %}
{% block before_html %}{% endblock %}
<!DOCTYPE html>
<html>
    <head>
        <meta charset="{{ _charset }}" />
        <meta name="robots" content="noindex,nofollow" />
        <meta name="viewport" content="width=device-width,initial-scale=1" />
        <title>{% block title %}{% endblock %}</title>
        <link rel="icon" type="image/png" href="{{ include('@Twig/images/favicon.png.base64') }}">
        <style>{{ include('@Twig/exception.css.twig') }}</style>
        {% block head %}{% endblock %}
    </head>
    <body>
        <header>
            <div class="container">
                <h1 class="logo">{{ include('@Twig/images/symfony-logo.svg') }} Symfony Exception</h1>

                <div class="help-link">
                    <a href="https://symfony.com/doc/{{ constant('Symfony\\Component\\HttpKernel\\Kernel::VERSION') }}/index.html">
                        <span class="icon">{{ include('@Twig/images/icon-book.svg') }}</span>
                        <span class="hidden-xs-down">Symfony</span> Docs
                    </a>
                </div>

                <div class="help-link">
                    <a href="https://symfony.com/support">
                        <span class="icon">{{ include('@Twig/images/icon-support.svg') }}</span>
                        <span class="hidden-xs-down">Symfony</span> Support
                    </a>
                </div>
            </div>
        </header>

        {% block body %}{% endblock %}
        {{ include('@Twig/base_js.html.twig') }}
    </body>
</html>
{% block after_html %}{% endblock %}
