{% extends "base.html.twig" %}

{% block body %}

    {% if error %}
        <div>{{ error.message }}</div>
    {% endif %}

    <form action="{{ path('form_login_check') }}" method="post">
        <label for="username">Username:</label>
        <input type="text" id="username" name="_username" value="{{ last_username }}" />

        <label for="password">Password:</label>
        <input type="password" id="password" name="_password" />

        <input type="hidden" name="_target_path" value="" />

        <input type="submit" name="login" />
    </form>

{% endblock %}
