<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Component\PropertyInfo\Tests\Fixtures;

/**
 * @author Tales Santos <tales.augusto.santos@gmail.com>
 */
class DefaultValue
{
    public $defaultInt = 30;
    public $defaultFloat = 30.5;
    public $defaultString = 'foo';
    public $defaultArray = [];
    public $defaultNull = null;
}
