<table class="trace trace-as-text">
    <thead class="trace-head">
        <tr>
            <th class="sf-toggle" data-toggle-selector="#trace-text-{{ index }}" data-toggle-initial="{{ 1 == index ? 'display' }}">
                <h3 class="trace-class">
                    {% if num_exceptions > 1 %}
                        <span class="text-muted">[{{ num_exceptions - index + 1 }}/{{ num_exceptions }}]</span>
                    {% endif %}
                    {{ exception.class|split('\\')|last }}
                    <span class="icon icon-close">{{ include('@Twig/images/icon-minus-square-o.svg') }}</span>
                    <span class="icon icon-open">{{ include('@Twig/images/icon-plus-square-o.svg') }}</span>
                </h3>
            </th>
        </tr>
    </thead>

    <tbody id="trace-text-{{ index }}">
        <tr>
            <td>
                {% if exception.trace|length %}
                <pre class="stacktrace">
                {%- apply escape('html') -%}
                    {{- include('@Twig/Exception/traces.txt.twig', { exception: exception, format: 'html' }, with_context = false) }}
                {%- endapply -%}
                </pre>
                {% endif %}
            </td>
        </tr>
    </tbody>
</table>
