<div id="header">
    <div class="container">
        <h1>{{ include('@WebProfiler/Icon/symfony.svg') }} Symfony <span>Profiler</span></h1>

        <div class="search">
            <form method="get" action="https://symfony.com/search" target="_blank">
                <div class="form-row">
                    <input name="q" id="search-id" type="search" placeholder="search on symfony.com">
                    <button type="submit" class="btn">Search</button>
                </div>
           </form>
        </div>
    </div>
</div>
