<div id="sidebar-search" class="hidden">
    <form action="{{ path('_profiler_search') }}" method="get">
        <div class="form-group">
            <label for="ip">IP</label>
            <input type="text" name="ip" id="ip" value="{{ ip }}">
        </div>

        <div class="form-group">
            <label for="method">Method</label>
            <select name="method" id="method">
                <option value="">Any</option>
                {% for m in ['DELETE', 'GET', 'HEAD', 'PATCH', 'POST', 'PUT'] %}
                    <option {{ m == method ? 'selected="selected"' }}>{{ m }}</option>
                {% endfor %}
            </select>
        </div>

        <div class="form-group">
            <label for="status_code">Status</label>
            <input type="number" name="status_code" id="status_code" min="100" max="599" value="{{ status_code }}">
        </div>

        <div class="form-group">
            <label for="url">URL</label>
            <input type="text" name="url" id="url" value="{{ url }}">
        </div>

        <div class="form-group">
            <label for="token">Token</label>
            <input type="text" name="token" id="token" value="{{ token }}">
        </div>

        <div class="form-group">
            <label for="start">From</label>
            <input type="date" name="start" id="start" value="{{ start }}">
        </div>

        <div class="form-group">
            <label for="end">Until</label>
            <input type="date" name="end" id="end" value="{{ end }}">
        </div>

        <div class="form-group">
            <label for="limit">Results</label>
            <select name="limit" id="limit">
                {% for l in [10, 50, 100] %}
                    <option {{ l == limit ? 'selected="selected"' }}>{{ l }}</option>
                {% endfor %}
            </select>
        </div>

        <div class="form-group">
            <button type="submit" class="btn btn-sm">Search</button>
        </div>
    </form>
</div>
