{% deprecated 'The template "' ~ _self ~'" is deprecated since Symfony 4.4, will be removed in 5.0.' %}
<?xml version="1.0" encoding="{{ _charset }}" ?>

<error code="{{ status_code }}" message="{{ status_text }}" />
