<?php

return [
    'Entry1' => [
        'Array' => [
            0 => 'foo',
            1 => 'bar',
        ],
        'Integer' => 5,
        'Boolean' => false,
        'Float' => 1.23,
    ],
    'Entry2' => 'String',
    'Traversable' => [
        'Foo' => 'Bar',
    ],
];
