{% extends '@WebProfiler/Profiler/base.html.twig' %}

{% block head %}
    <style>
        {{ include('@WebProfiler/Profiler/open.css.twig') }}
    </style>
{% endblock %}

{% block body %}
    {% set source = filename|file_excerpt(line, -1) %}
    <div class="header">
        <h1>{{ file }}{% if 0 < line %} <small>line {{ line }}</small>{% endif %}</h1>
        <a class="doc" href="https://symfony.com/doc/{{ constant('Symfony\\Component\\HttpKernel\\Kernel::VERSION') }}/reference/configuration/framework.html#ide" rel="help">Open in your IDE?</a>
    </div>
    <div class="source">
        {% if source is null %}
            <p class="empty">The file is not readable.</p>
        {% else %}
            {{ source|raw }}
        {% endif %}
    </div>
{% endblock %}
