{% extends '@WebProfiler/Profiler/base.html.twig' %}

{% block title 'Redirection Intercepted' %}

{% block body %}
    <div class="sf-reset">
        <div class="block-exception">
            <h1>This request redirects to <a href="{{ location }}">{{ location }}</a>.</h1>

            <p>
                <small>
                    The redirect was intercepted by the web debug toolbar to help debugging.
                    For more information, see the "intercept-redirects" option of the Profiler.
                </small>
            </p>
        </div>
    </div>
{% endblock %}
