<?php

/*
 * This file is part of the Symfony package.
 *
 * (c) Fabien Potencier <fabien@symfony.com>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */

namespace Symfony\Bridge\Doctrine\Tests\Fixtures\Type;

class StringWrapper
{
    private $string;

    public function __construct(string $string = null)
    {
        $this->string = $string;
    }

    public function getString(): string
    {
        return $this->string;
    }
}
